"use strict";

//System and File I/O modules
var Sys = require("../System");
var Util = Sys.Util;
var EventEmitter = Sys.EventEmitter;

/**
  * A client class using the HTTP protocol for communcation between the
  * game engine and the user
  * @name HTTPServer.Client
  * @class
  * @extends EventEmitter
  * @property {String} remoteAddress The remote client address
  */
var HTTPClient = Object.create(EventEmitter.prototype, Util.toPropertiesObject(
/** @lends HTTPServer.Client# */
{
	/**
	 * The Socket.IO socket object associated with this client
	 * @private
	 */
	_socket: null

	/**
	 * The HTTPServer object associated with this client
	 * @private
	 */
	, _server: null

	/**
	 * The unique ID for this client
	 */
	, id: null

	,_bindEventHandlers: function()
	{
		//Bind socket events
		var self = this;
		var socket = this._socket;
		socket.on("message", function(message)
		{
			self.emit("message", message);
		});

		socket.on("disconnect", function()
		{
			self._socket == null;
			self.disconnect();
		});
	}

	/**
	  * Sends a message to the user
	  * @param {Object|String} message The message to be sent to the user.
	  */
	, send: function(message, data)
	{
		if(this.isConnected)
			this._socket.json.send({message: message, data: data});
	}

	/**
	  * Closes the connection to the user
	  */
	, disconnect: function()
	{
		if(this.isConnected)
		{
			var socket = this._socket;
			this._socket = null;

			socket.removeAllListeners();
			socket.disconnect();
		}
		this.emit("disconnected");
	}
}));

module.exports =
{
	create: function(server, socket)
	{
		var client = Object.create(HTTPClient, Util.toPropertiesObject(
			{
				_socket: socket
				, _server: server
			}));

		Object.defineProperty(client, "id",
			{
				get: function()
				{
					if(!this._socket)
						return undefined;

					return this._socket.sessionId;
				}
				, enumerable: true
				, configurable: false
			})

		Object.defineProperty(client, "isConnected",
			{
				get: function()
				{
					return (this._socket != null);
				}
				, enumerable: true
			});

		Object.defineProperty(client, "remoteAddress",
			{
				get: function()
				{
					var request = this._socket.request;
					var ip = request.socket.remoteAddress;

					//Check to see if this server is actually behind a proxy,
					// which is a common setup when running on a non-80 port but
					// exposed through an external web-server proxy.
					var proxy_ip = request.headers["x-forwarded-for"];
					if(ip == "127.0.0.1" && proxy_ip)
						ip = proxy_ip;

					return ip;
				}
				, enumerable: true
				, configurable: false
			});

		client._bindEventHandlers();
		return client;
	}
}
