"use strict";

var Util = require("./System").Util;

/**
  * A message represents a single action/command/task to be performed by the
  * MUD.  All messages must inherit from this object
  * @class
  * @name Message
 */
var Message =
/** @lends Message# */
{
	/**
	 * The source of the message.  This can be a system process or an in game
	 * object
	 * @type Object
	 */
	source: null

	/**
	 * The actual message
	 * @type String
	 */
	, message: null

	/**
	 * Any data associated with the message
	 */
	, data: null

	/**
	 * The date/time the message was created
	 */
	, createdDate: new Date()

	/**
	 * Creates a basic message object that can be sent to the Message Pump
	 */
	, create: function(source, message, data)
	{
		var msg = Object.create(this, Util.toPropertiesObject(
			{
				source: source
				, message: message
				, data: data
				, createdDate: new Date()
			}));

		return msg;
	}
};

module.exports = Message;