/**
  * The base system framework API in a single spot so that
  * the individual components do not need to require'ed.
  * This also abstracts the actual libraries used in case
  * the core system changes
  * @namespace
  * @name System
  */
var System = 
{
	/**
	  * Provides general utility functionality like console printing
	  * and object inspection.
	  */
	Util: require("util")
	
	/**
	  * Constructor for the Event Emitter object
	  */
	, EventEmitter: require("events").EventEmitter
	
	/**
	  * Provides file system functionality
	  */
	, FileSystem: require("fs")
	
	/**
	  * Provides basic path functionality
	  */
	, Path: require("path")
	
	/**
	  * Provides HTTP Server functionality
	  */
	, HTTP: require("http")
	
	/**
	  * Provides URL parsing/processing functionality
	  */
	, URL: require("url")
	
	/**
	  * Provides real-time/bi-directional HTTP functionality
	  */
	, SocketIO: require("socket.io")
}

//Extend 'Utul' with the toPropertiesObject method

/**
 * Converts a normal object into a properties object that can be used
 * with Object.create or Object.defineProperties.  It has logic to also
 * set the enumerable property if the method starts with an underscore '_'
 * @memberOf System.Util
 * @inner
 */
System.Util.toPropertiesObject = function(object)
{
	var props_obj = {};
	
	if(!object)
		return props_obj;

	var keys = Object.getOwnPropertyNames(object);
	var key = null;
	while(keys.length)
	{
		key = keys.shift();

		//Member names define the property attributes
		var enumerable = (key !== "constructor")
			, configurable = true
			, writable = true;
		enumerable &= (key.charAt(0) !== "_");

		//Add the property to the properies object using the new name
		props_obj[key] =
		{
			enumerable: enumerable
			, configurable: configurable
			, writable: writable
			, value: object[key]
		};
	}
	
	return props_obj;
}

module.exports = System;